@@ -6,12 +6,23 @@ import re
 from dataclasses import dataclass, field
 from inspect import getmembers, isclass, isroutine
 from typing import (
-    Any, Dict, Generic, Iterable, List, NamedTuple, Optional,
-    Sequence, Set, Tuple, Type, TypeVar, Union, get_type_hints
+    Any,
+    Dict,
+    Iterable,
+    List,
+    NamedTuple,
+    Optional,
+    Sequence,
+    Set,
+    Tuple,
+    Type,
+    TypeVar,
+    Union,
+    get_type_hints
 )
 
-import yaml
 from .exceptions import PlaceholderVariableError
+from .parser import YamlParser
 from .utils import MISSING, hybridmethod, OPTIONAL
 
 
@@ -21,12 +32,6 @@ LOGGER = logging.getLogger(__name__)
 
 # Type declaration for classes
 T = TypeVar("T")
-CN = TypeVar('CN', bound='ConfigNode')
-CV = TypeVar('CV', bound='ConfigPlaceholderVar')
-CP = TypeVar('CP', bound='ConfigParam')
-CR = TypeVar('CR', bound='ConfigNodeProps')
-CE = TypeVar('CE', bound='ConfigRegistryReference')
-PS = TypeVar('PS', bound='ParameterSpec')
 
 # Constants
 CONFIG_NODE_PROPERTIES_NAME = '__node__'
@@ -47,22 +52,23 @@ class ParameterSpec(NamedTuple):
     type: Type[T]
 
     @classmethod
-    def from_string(cls: Type[PS], string: str) -> PS:
+    def from_string(cls: Type['ParameterSpec'],
+                    string: str) -> 'ParameterSpec':
         if not re.findall(CONFIG_PARAM_CLI_FORMAT, string):
             msg = f"Cannot parse string '{string}' into a parameter"
             raise ValueError(msg)
         name, default = string.split('=', 1)
-        default = yaml.safe_load(default)
+        default = YamlParser.load(default)
         return cls(name=name, default=default, type=type(default))
 
-    def to_dict(self: PS) -> dict:
+    def to_dict(self: 'ParameterSpec') -> dict:
         outdict = None
         for part in self.name.split('.')[::-1]:
             outdict = {part: outdict} if outdict else {part: self.default}
         return outdict
 
 
-class ConfigParam(Generic[CP]):
+class ConfigParam:
     def __init__(self, target_type):
         self.type = target_type
 
@@ -79,10 +85,10 @@ class OptionalConfigParam(ConfigParam):
     ...
 
 
-class ConfigRegistryReference(Generic[CE]):
+class ConfigRegistryReference:
     """Extract registry references and resolve them"""
     def __init__(
-        self: CE,
+        self: 'ConfigRegistryReference',
         param_name: str = None,
         registry_ref: str = None,
         registry_args: Sequence[Any] = None
@@ -91,7 +97,7 @@ class ConfigRegistryReference(Generic[CE]):
         self.registry_ref = registry_ref
         self.registry_args = registry_args or []
 
-    def name_as_placeholder_variable(self: CE):
+    def name_as_placeholder_variable(self: 'ConfigRegistryReference'):
         """Split the name (if provided) to be used
         as the path for placeholder variable"""
         if self.param_name:
@@ -99,7 +105,8 @@ class ConfigRegistryReference(Generic[CE]):
         return None
 
     @classmethod
-    def from_str(cls: Type[CE], string: str) -> CE:
+    def from_str(cls: Type['ConfigRegistryReference'],
+                 string: str) -> 'ConfigRegistryReference':
         """Extract name and parameters of the reference from a string"""
 
         args = string.split(CONFIG_NODE_REGISTRY_SEPARATOR)
@@ -112,14 +119,16 @@ class ConfigRegistryReference(Generic[CE]):
                        registry_ref=registry_ref,
                        registry_args=args)
 
-    def resolve_later(self: CE, *args, **kwargs) -> CE:
+    def resolve_later(self: 'ConfigRegistryReference',
+                      *args,
+                      **kwargs) -> 'ConfigRegistryReference':
         copy = deepcopy(self)
         copy.resolve = functools.partial(copy.resolve, *args, **kwargs)
         return copy
 
-    def resolve(self: CE,
+    def resolve(self: 'ConfigRegistryReference',
                 *args,
-                node_cls: Sequence[CN] = None,
+                node_cls: Sequence['ConfigNode'] = None,
                 **kwargs) -> Any:
         """Resolves and instantiates a reference to a registry object using
         `param_value`. `node_cls` can be a list of ConfigNode objects to
@@ -137,7 +146,7 @@ class ConfigRegistryReference(Generic[CE]):
         if node_cls:
             if isclass(node_cls) and not issubclass(node_cls, ConfigNode):
                 msg = ('The registry reference resolver has receive an object'
-                        f'of type {node_cls}, which is not a ConfigNode')
+                       f'of type {node_cls}, which is not a ConfigNode')
                 raise ValueError(msg)
 
             # we operate slightly differently depending if the registry
@@ -164,8 +173,11 @@ class ConfigRegistryReference(Generic[CE]):
         return CONFIG_NODE_REGISTRY_SEPARATOR in string
 
 
-class ConfigNodeProps(Generic[CR]):
-    def __init__(self: CR, node: CN, name: str, parent: CN = None):
+class ConfigNodeProps:
+    def __init__(self: 'ConfigNodeProps',
+                 node: 'ConfigNode',
+                 name: str,
+                 parent: 'ConfigNode' = None):
         self.node = node
         self.config_vars = []
         self.param_keys = set()
@@ -182,19 +194,21 @@ class ConfigNodeProps(Generic[CR]):
         return self.node_cls.__name__
 
     @classmethod
-    def get_props(cls: Type[CR], node: CN) -> CR:
+    def get_props(cls: Type['ConfigNodeProps'],
+                  node: 'ConfigNode') -> 'ConfigNodeProps':
         return getattr(node, CONFIG_NODE_PROPERTIES_NAME)
 
-    def __set_props(self: CR):
+    def __set_props(self: 'ConfigNodeProps'):
         """Binds this property file to a node."""
         setattr(self.node, CONFIG_NODE_PROPERTIES_NAME, self)
 
-    def set_parent(self: CR, parent_node: Union[CN, None]):
+    def set_parent(self: 'ConfigNodeProps',
+                   parent_node: Union['ConfigNode', None]):
         # keep track of which is the root node for this config
         self.parent = self.node if parent_node is None else parent_node
 
-    def set_name(self: CR, node_name: Union[str, None]):
-            # set the full path to this node as its name
+    def set_name(self: 'ConfigNodeProps', node_name: Union[str, None]):
+        # set the full path to this node as its name
         self.short_name = self.cls_name if node_name is None else node_name
         if self.is_root():
             self.long_name = self.short_name
@@ -204,40 +218,77 @@ class ConfigNodeProps(Generic[CR]):
 
     @hybridmethod
     def get_annotations(
-        cls: Type[CR],
-        node_cls: Type[CN]
+        cls: Type['ConfigNodeProps'],
+        node_cls: Type['ConfigNode'],
+        traverse: bool = True
     ) -> Dict[str, ConfigParam]:
+
+        # traverse the inheritance tree if we are asked to; don't
+        # otherwise (that is, only return type annotations for this class.)
+        raw_annotations = (get_type_hints(node_cls) if traverse else
+                           getattr(node_cls, '__annotations__', {}))
+
         # these are all annotations for parameters for this node; we use
         # them to cast param values to the right type as we parse a config
-        annotations = {name: annotation for name, annotation in
-                       get_type_hints(node_cls).items()
+        annotations = {name: annotation
+                       for name, annotation in raw_annotations.items()
                        if isinstance(annotation, ConfigParam)}
         return annotations
 
     @get_annotations.instancemethod
-    def get_annotations(self: CR) -> Dict[str, ConfigParam]:
-        return type(self).get_annotations(self.node_cls)
+    def get_annotations(
+        self: 'ConfigNodeProps',
+        traverse: bool = True
+    ) -> Dict[str, ConfigParam]:
+        return type(self).get_annotations(node_cls=self.node_cls,
+                                          traverse=traverse)
 
     @hybridmethod
-    def get_defaults(cls: Type[CR], node_cls: Type[CN]) -> Dict[str, Any]:
+    def get_defaults(cls: Type['ConfigNodeProps'],
+                     node_cls: Type['ConfigNode']) -> Dict[str, Any]:
         # These are all the default values that have been provided
         # for the parameters for this node.
 
+        # Calling with traverse == True returns all inherited
+        # and future type hints.
+        all_annotations = cls.get_annotations(node_cls, traverse=True)
+
+        # these are local annotations, i.e. annotations defined in this
+        # NodeConfig only.
+        local_annotations = cls.get_annotations(node_cls, traverse=False)
+
+        # This is a bit confusing but: sometimes a default
+        # is REMOVED when a new ConfigNode is created from a
+        # old config node. The use case is sample: you might want
+        # to make previously default parameters not default anymore.
+        # In that case, if the parameter is part of the
+        # local annotations, but not part of the values. (note
+        # we can't use hasattr() because that also resolves inheritance,
+        # so we use in vars() instead).
+        removed_defaults = {param_name for param_name in local_annotations
+                            if param_name not in vars(node_cls)}
+
         # We use getmembers instead of __dict__ because it
         # resolves inheritance (__dict__ does not!)
         all_members = dict(getmembers(node_cls))
 
+        # We are finally ready to compose defaults!
         defaults = {name: all_members[name]
-                    for name in cls.get_annotations(node_cls)
-                    if name in all_members}
+                    for name in all_annotations
+                    # has a default value somewhere in the MRO chain...
+                    if (name in all_members) and
+                    # but it wasn't explicitly removed.
+                    (name not in removed_defaults)}
+
         return defaults
 
     @get_defaults.instancemethod
-    def get_defaults(self: CR) -> Dict[str, Any]:
+    def get_defaults(self: 'ConfigNodeProps') -> Dict[str, Any]:
         return type(self).get_defaults(self.node_cls)
 
     @hybridmethod
-    def get_subnodes(cls: Type[CR], node_cls: Type[CN]) -> Dict[str, CN]:
+    def get_subnodes(cls: Type['ConfigNodeProps'],
+                     node_cls: Type['ConfigNode']) -> Dict[str, 'ConfigNode']:
         # these are almost all the subnodes for the node; for now, we
         # capture the ones that are of the form
         #   class node(ConfigNode):
@@ -257,26 +308,31 @@ class ConfigNodeProps(Generic[CR]):
         return subnodes
 
     @get_subnodes.instancemethod
-    def get_subnodes(self: CR) -> Dict[str, CN]:
+    def get_subnodes(self: 'ConfigNodeProps') -> Dict[str, 'ConfigNode']:
         return type(self).get_subnodes(self.node_cls)
 
     @hybridmethod
-    def get_all_cls_members(cls: Type[CR],
-                            node_cls: Type[CN]) -> Dict[str, Any]:
+    def get_all_cls_members(cls: Type['ConfigNodeProps'],
+                            node_cls: Type['ConfigNode']) -> Dict[str, Any]:
         all_non_routines = getmembers(node_cls, lambda a: not(isroutine(a)))
-        invalid_name_fn = lambda n: ((n.startswith('__') and n.endswith('__'))
-                                     or n == '_is_protocol')
+
+        def invalid_name_fn(fn_name: str):
+            return ((fn_name.startswith('__') and fn_name.endswith('__'))
+                    or fn_name == '_is_protocol')
+
         members = {name: value for name, value in all_non_routines
                    if not invalid_name_fn(name)}
         return members
 
     @get_all_cls_members.instancemethod
-    def get_all_cls_members(self: CR) -> Dict[str, Any]:
+    def get_all_cls_members(self: 'ConfigNodeProps') -> Dict[str, Any]:
         return type(self).get_all_cls_members(self.node_cls)
 
     @hybridmethod
-    def get_all_parameters(cls: Type[CR],
-                           node_cls: Type[CN]) -> Sequence[ParameterSpec]:
+    def get_all_parameters(
+        cls: Type['ConfigNodeProps'],
+        node_cls: Type['ConfigNode']
+    ) -> Sequence[ParameterSpec]:
         """Get all parameters for this node as well as for its subnodes."""
 
         all_parameters = []
@@ -303,7 +359,7 @@ class ConfigNodeProps(Generic[CR]):
         return all_parameters
 
     @get_all_parameters.instancemethod
-    def get_all_parameters(self: CR) -> Sequence[ParameterSpec]:
+    def get_all_parameters(self: 'ConfigNodeProps') -> Sequence[ParameterSpec]:
         return type(self).get_all_parameters(self.node_cls)
 
     def assign_param(self, name: str, value: Any, annotation: Any = None):
@@ -322,13 +378,13 @@ class ConfigNodeProps(Generic[CR]):
                 setattr(self.node, '__annotations__', {})
             setattr(self.node.__annotations__, name, annotation)
 
-    def get_params_names(self: CR) -> Sequence[str]:
+    def get_params_names(self: 'ConfigNodeProps') -> Sequence[str]:
         return tuple(self.param_keys)
 
-    def add_var(self: CR, var: CV):
+    def add_var(self: 'ConfigNodeProps', var: 'ConfigPlaceholderVar'):
         self.config_vars.append(var)
 
-    def pop(self: CR, key: str, default=MISSING) -> Any:
+    def pop(self: 'ConfigNodeProps', key: str, default=MISSING) -> Any:
         if key in self.param_keys:
             self.param_keys.remove(key)
             value = getattr(self.node, key)
@@ -339,10 +395,10 @@ class ConfigNodeProps(Generic[CR]):
         else:
             KeyError(f'`{key}` is not a parameter in {self.long_name}')
 
-    def is_root(self: CR) -> bool:
+    def is_root(self: 'ConfigNodeProps') -> bool:
         return self.parent == self.node
 
-    def get_root(self: CR) -> CN:
+    def get_root(self: 'ConfigNodeProps') -> 'ConfigNode':
         """Traverse the configuration this node is part of to
         find the root node"""
         if self.is_root():
@@ -350,7 +406,8 @@ class ConfigNodeProps(Generic[CR]):
         else:
             return self.get_props(self.parent).get_root()
 
-    def get_children(self: CN, recursive: bool = False) -> Iterable[CN]:
+    def get_children(self: 'ConfigNode',
+                     recursive: bool = False) -> Iterable['ConfigNode']:
         """Get a iterable of all the subnodes to this config node"""
         for key, value in sorted(self.node):
             if isinstance(value, ConfigNode):
@@ -360,7 +417,7 @@ class ConfigNodeProps(Generic[CR]):
                     for ch_key, ch_value in ch_sub:
                         yield (f'{key}.{ch_key}', ch_value)
 
-    def apply_vars(self: CR) -> Set[str]:
+    def apply_vars(self: 'ConfigNodeProps') -> Set[str]:
         """Resolve all variables by applying substitutions to its
         parameters or its subnodes'. Raises a RuntimeError in case
         of a cyclical dependency."""
@@ -377,23 +434,24 @@ class ConfigNodeProps(Generic[CR]):
                    'due to cyclical assignments. Check your config!')
             raise RuntimeError(msg)
 
-    def to_dict(self: CR) -> Dict[str, Any]:
+    def to_dict(self: 'ConfigNodeProps') -> Dict[str, Any]:
         return {k: (self.get_props(v).to_dict()
                     if isinstance(v, ConfigNode) else
                     (repr(v) if isinstance(v, ConfigPlaceholderVar) else v))
                 for k, v in self.node}
 
-    def to_json(self: CR,
+    def to_json(self: 'ConfigNodeProps',
                 *args: Sequence[Any],
                 **kwargs: Dict[str, Any]) -> str:
         return json.dumps(self.to_dict(), *args, **kwargs)
 
-    def to_yaml(self: CR,
+    def to_yaml(self: 'ConfigNodeProps',
                 *args: Sequence[Any],
                 **kwargs: Dict[str, Any]) -> str:
-        return yaml.safe_dump(self.to_dict(), *args, **kwargs)
+        return YamlParser.dump(self.to_dict(), *args, **kwargs)
 
-    def validate_subnodes(self: CR, subnodes_to_validate: Sequence[str]):
+    def validate_subnodes(self: 'ConfigNodeProps',
+                          subnodes_to_validate: Sequence[str]):
         """Check that if specified subnodes are instances of ConfigNode;
         used during pickling/unpickling and copying to make sure nothing
         went bad."""
@@ -414,10 +472,10 @@ class ConfigNodeProps(Generic[CR]):
                     raise ValueError(msg)
 
     @hybridmethod
-    def merge_nodes(cls: Type[CR],
-               node: CN,
-               other_node: CN,
-               flex: bool = False) -> CN:
+    def merge_nodes(cls: Type['ConfigNodeProps'],
+                    node: 'ConfigNode',
+                    other_node: 'ConfigNode',
+                    flex: bool = False) -> 'ConfigNode':
         """Create a new node node where values form `node`
         are updated with values from `other_node`."""
 
@@ -438,21 +496,21 @@ class ConfigNodeProps(Generic[CR]):
         return node_cls(config_values, __flex_node__=flex)
 
     @merge_nodes.instancemethod
-    def merge_nodes(self: CR,
-                    other_node: CN,
-                    flex: bool = False) -> CN:
+    def merge_nodes(self: 'ConfigNodeProps',
+                    other_node: 'ConfigNode',
+                    flex: bool = False) -> 'ConfigNode':
         return type(self).merge_nodes(node=self.node,
                                       other_node=other_node,
                                       flex=flex)
 
 
-class ConfigNode(Generic[CN]):
+class ConfigNode:
     """A generic configuration node."""
 
     def __init__(
-        self: CN,
-        config: Optional[Union[Dict[str, Any], CN]] = None,
-        __parent__ : CN = None,
+        self: 'ConfigNode',
+        config: Optional[Union[Dict[str, Any], 'ConfigNode']] = None,
+        __parent__: 'ConfigNode' = None,
         __flex_node__: bool = False,
         __name__: str = None,
         **kwargs: Dict[str, Any],
@@ -511,10 +569,10 @@ class ConfigNode(Generic[CN]):
         # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ PHASE 1 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
         # Override with options that are provided in `config` dictionary
         for param_name, param_value in config.items():
-            # a little lambda function to log how we are doing
-            debug_call = lambda case: LOGGER.debug(
-                f'[PHASE 1][{node_props.long_name}][{case}] {param_name}'
-            )
+            # a little function to log how we are doing
+            def debug_call(case: str):
+                LOGGER.debug(f'[PHASE 1][{node_props.long_name}][{case}]'
+                             f'{param_name}')
 
             if ConfigRegistryReference.contains(param_name):
                 # we have a registry reference! we need to extract the
@@ -526,9 +584,10 @@ class ConfigNode(Generic[CN]):
                 registry_reference = None
 
             # check if it is a valid parameter, if not, raise a KeyError
-            if ((param_name not in annotations) and
-                (param_name not in subnodes) and
-                not(__flex_node__)):
+            is_supported_parameter = (param_name not in annotations and
+                                      param_name not in subnodes and
+                                      not __flex_node__)
+            if is_supported_parameter:
                 debug_call('0.not_supported')
                 # CASE 0: we can't add this key to this config; raise an error.
                 msg = (f'Parameter "{param_name}" not '
@@ -554,7 +613,7 @@ class ConfigNode(Generic[CN]):
                     debug_call('1.S1.1.registry+submodule')
                     node_cls_to_merge = subnodes[param_name]
                 elif (param_name in annotations and
-                    issubclass(annotations[param_name].type, ConfigNode)):
+                      issubclass(annotations[param_name].type, ConfigNode)):
                     # CASE 1.S1.2: we also need to merge with the type we also
                     #              get from the parameter annotation. This is
                     #              usually empty, but we merge with it for
@@ -625,7 +684,8 @@ class ConfigNode(Generic[CN]):
                 # parameter is, so we first extract the type.
                 param_type = annotations[param_name].type
 
-                if (isclass(param_type) and issubclass(param_type, ConfigNode)):
+                if isclass(param_type) \
+                        and issubclass(param_type, ConfigNode):
                     # CASE 3.1: sometimes the type of a paramemter is
                     #           ConfigNode or a subclass of it, making
                     #           necessary to pass some extra parameters to
@@ -643,7 +703,9 @@ class ConfigNode(Generic[CN]):
                         )
                     else:
                         debug_call('3.1.2.ann_subnode/skip_cast')
-                        param_type = lambda x: x
+
+                        def param_type(x):
+                            return x
                 elif isinstance(param_value, param_type):
                     # We cast to param_type, but only if we absolutely
                     # have to. This prevents unwanted initializations is
@@ -659,7 +721,9 @@ class ConfigNode(Generic[CN]):
                     # if the object is already of the expected type should
                     # fix this).
                     debug_call('3.2.ann_skip_casting')
-                    param_type = lambda x: x
+
+                    def param_type(x):
+                        return x
                 else:
                     # this is the case where we cast
                     debug_call('3.3.ann_do_cast')
@@ -724,8 +788,8 @@ class ConfigNode(Generic[CN]):
 
             if is_missing_default and is_optional_default:
                 msg = (f'[PHASE 2][CLS {type(self).__name__}] '
-                        f'Skipping {param_name} because it is '
-                        f'annotated as {OptionalConfigParam.__name__}')
+                       f'Skipping {param_name} because it is '
+                       f'annotated as {OptionalConfigParam.__name__}')
                 LOGGER.debug(msg)
             elif is_missing_default:
                 # This parameter was not overwritten by the configuration
@@ -793,18 +857,18 @@ class ConfigNode(Generic[CN]):
 
         LOGGER.debug(f'\n[DONE NODE INIT][CLS {type(self).__name__}]\n')
 
-    def __iter__(self: CN) -> Iterable[Tuple[str, Any]]:
+    def __iter__(self: 'ConfigNode') -> Iterable[Tuple[str, Any]]:
         """Get a iterable of names and parameters in this node,
         including subnodes"""
         yield from ((k, self[k]) for k in
                     ConfigNodeProps.get_props(self).get_params_names())
 
-    def __repr__(self: CN):
+    def __repr__(self: 'ConfigNode'):
         full_name = ConfigNodeProps.get_props(self).long_name
         return (f'<{full_name} config at {hex(id(self))} '
                 f'with params {ConfigNodeProps.get_props(self).to_dict()}>')
 
-    def __contains__(self: CN,  key: str) -> bool:
+    def __contains__(self: 'ConfigNode',  key: str) -> bool:
         # allows to check for nested configs!
         key, *rest = key.split('.')
         rest = '.'.join(rest)
@@ -839,14 +903,14 @@ class ConfigNode(Generic[CN]):
             msg = f'`{key}` is not a parameter in {node_props.long_name}'
             raise KeyError(msg)
 
-    def __len__(self: CN) -> int:
+    def __len__(self: 'ConfigNode') -> int:
         return sum(1 for _ in self)
 
-    def __rshift__(self: CN, other: CN) -> CN:
+    def __rshift__(self: 'ConfigNode', other: 'ConfigNode') -> 'ConfigNode':
         # other >> self
         return ConfigNodeProps.get_props(other).merge_nodes(self)
 
-    def __lshift__(self: CN, other: CN) -> CN:
+    def __lshift__(self: 'ConfigNode', other: 'ConfigNode') -> 'ConfigNode':
         # other << self
         return other >> self
 
@@ -860,20 +924,20 @@ class VarMatch:
     )
 
 
-class ConfigPlaceholderVar(Generic[CV]):
+class ConfigPlaceholderVar:
     """A Placeholder for config values that contains a variable.
     Placeholder variables are specified using the following format:
 
         PLACEHOLDER := ${PATH_TO_VALUE}
         PATH_TO_VALUE := PATH_TO_VALUE.VAR_NAME
-        VAR_NAME := [a-zA-Z_]\w+
+        VAR_NAME := [a-zA-Z_]\\w+
 
     PATH_TO_VALUE is used to traverse down the config node
     to find suitable replacement values for the placeholder.
     """
 
     def __init__(
-        self: CV,
+        self: 'ConfigPlaceholderVar',
         parent_node: ConfigNode,
         param_name: str,
         param_value: str,
@@ -889,10 +953,8 @@ class ConfigPlaceholderVar(Generic[CV]):
                                   else lambda x: x)
         self.placeholder_vars = []
 
-        # import here to avoid cycles
-        from espresso_config.registry import ConfigRegistry
-
-        for match in re.finditer(CONFIG_PLACEHOLDER_VAR_TEMPLATE, param_value):
+        for match in re.finditer(CONFIG_PLACEHOLDER_VAR_TEMPLATE,
+                                 param_value):
             var_path_and_registry = match.group()[2:-1]
             if var_path_and_registry == '':
                 # CASE 0: the regex matches even in cases the variable
@@ -909,13 +971,13 @@ class ConfigPlaceholderVar(Generic[CV]):
                          match=match.group())
             )
 
-    def __repr__(self: CV) -> str:
+    def __repr__(self: 'ConfigPlaceholderVar') -> str:
         return self.__str__()
 
-    def __str__(self: CV) -> str:
+    def __str__(self: 'ConfigPlaceholderVar') -> str:
         return str(self.unresolved_param_value)
 
-    def resolve(self: CV):
+    def resolve(self: 'ConfigPlaceholderVar'):
         """Replace variables in this value with their actual value"""
 
         # we need to start looking from the root
@@ -946,11 +1008,12 @@ class ConfigPlaceholderVar(Generic[CV]):
             # apply_vars to make sure that all substitutions in
             # the subnode are gracefully handled.
             if isinstance(placeholder_substitution, ConfigNode):
-                ConfigNodeProps.get_props(placeholder_substitution).apply_vars()
+                ConfigNodeProps.get_props(placeholder_substitution)\
+                    .apply_vars()
 
             # trick of the century: sometimes, by doing variable resolution,
-            # we end up with another variable! in that case, we simply
-            # tell the parent node with the class to do variable resolution too.
+            # we end up with another variable! in that case, we simply tell
+            # the parent node with the class to do variable resolution too.
             if isinstance(placeholder_substitution, ConfigPlaceholderVar):
                 # asking the parent to apply all its vars!
                 ConfigNodeProps.get_props(
@@ -1017,7 +1080,7 @@ class ConfigPlaceholderVar(Generic[CV]):
         setattr(self.parent_node, self.param_name, replaced_value)
 
     @classmethod
-    def contains(cls: Type[CV], value: Any) -> bool:
+    def contains(cls: Type['ConfigPlaceholderVar'], value: Any) -> bool:
         """Returns True if value has variable (${...})
            somewhere, False otherwise"""
         return (isinstance(value, str) and
@@ -1027,7 +1090,7 @@ class ConfigPlaceholderVar(Generic[CV]):
 class ConfigFlexNode(ConfigNode):
     """Just like a ConfigNode, except it allows for additional
     parameters other than the ones provided with annotations"""
-    def __init__(self: CN,
+    def __init__(self: 'ConfigNode',
                  *args: List[Any],
                  __flex_node__: bool = True,
                  **kwargs: Dict[str, Any]) -> None:
